using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Elementals
{
    [ModuleInfo("", "Sends through data from one of the two inputs based on the control parameter/input", CategoryType.Generic, CategoryData.Generic, IconName="Junction.ico")]
    class GenericJunction:Module
    {
        private int control = 0;

        public int Control
        {
            get { return control; }
            set { control = value;
        }
        }

        public GenericJunction()
        {
            AddInPort("Input 0", typeof(IDataObject), true, false);
            AddInPort("Input 1", typeof(IDataObject), true, false);
            AddInPort("Control", typeof(Integer), false, true);

            p0 = GetInPort("Input 0");
            p1 = GetInPort("Input 1");

            this.PortConnected += new PortConnectedEventHandler(GenericJunction_PortConnecting);
            this.PortDisconnected += new PortDisconnectedEventHandler(GenericJunction_PortDisconnecting);
        }

        private InPort p0, p1;
        private OutPort o;

        void GenericJunction_PortDisconnecting(object source, OutPort portFrom, InPort portTo)
        {
            if (portTo == p0)
            {
                p0 = null;
                if (p1 == null)
                {
                    RemoveOutPort("Output");
                    p1.PortType = typeof(IDataObject);
                    p0.PortType = typeof(IDataObject);
                }
            }

            if (portTo == p1)
            {
                p1 = null;
                if (p0 == null)
                {
                    RemoveOutPort("Output");
                    p1.PortType = typeof(IDataObject);
                    p0.PortType = typeof(IDataObject);
                }
            }
        }

        void GenericJunction_PortConnecting(object source, OutPort portFrom, InPort portTo)
        {
            if (this.o == null)
            {
                this.AddOutPort("Output", portFrom.PortType);
                o = GetOutPort("Output");
            }
            
            this.p0.PortType = portFrom.PortType;
            this.p1.PortType = portFrom.PortType;
        }

        public override void Execute()
        {
            if (GetInPort("Control").Link != null)
            {
                Integer c = (Integer)GetInput("Control");
                control = c.Value;
            }

            if (control == 0)
            {
                UpdateInput("Input 0");
                SetOutput("Output", GetInput("Input 0"), IsDataSame("Input 0"));
            }
            else
            {
                UpdateInput("Input 1");
                SetOutput("Output", GetInput("Input 1"), IsDataSame("Input 1"));
            }
        }
    }
}
