using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;

namespace Examples
{
    /// <summary>
    /// Module shows how to use the Module.Write* method to output some information.
    /// <list type="table">
    /// <item><term><c>WriteConsole</c></term><description>outputs the message to Console window</description></item>
    /// <item><term><c>WriteDebug</c></term><description>outputs the message to Debug window</description></item>
    /// <item><term><c>WriteLog</c></term><description>outputs the message to Debug window</description></item>
    /// <item><term><c>Write</c></term><description>generic method</description></item>
    /// </list>
    /// The generic <c>Write</c> method can be called with different <c>WriteTargets</c> or predefined <c>Globals.DebugLevel*</c>.
    /// </summary>
    [ModuleInfo("Petr Vanecek", "Module shows how to use the Module.Write* method to .", CategoryType.Generic, CategoryData.Generic, Assemblied = "2007-06-25")]
    class DebugLevelExample : Zcu.Mve.Core.Module
    {
        public override void Execute()
        {
            // Fixed output target
            this.WriteConsole("Console message", "this message goes to console window");
            this.WriteDebug("Debug message", "this message goes to debug window");
            this.WriteLog("Log message", "this message goes to ");

            // Fixed output target using generic Write method
            this.Write(Globals.WriteTarget.Console,"Console message", "this message goes to console window");
            this.Write(Globals.WriteTarget.Debug, "Debug message", "this message goes to debug window");
            this.Write(Globals.WriteTarget.Log, "Log message", "this message goes to ");

            // Variable output target - depends on Globals.DebugLevel* value
            this.Write(Globals.DebugLevel0, "Debug level 0 message", "this message goes to " + Globals.DebugLevel0.ToString());
            this.Write(Globals.DebugLevel1, "Debug level 1 message", "this message goes to " + Globals.DebugLevel1.ToString());
            this.Write(Globals.DebugLevel2, "Debug level 2 message", "this message goes to " + Globals.DebugLevel2.ToString());
        }
    }
}
