using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;

namespace Examples
{
    class PointInterpolator:Module
    {
        bool paramsChanged = true;

		public enum interpolator {Linear, Cosine};

		interpolator interp;

		public interpolator Interpolator
		{
			get { return interp; }
			set { interp = value; }
		}

        double mix = 0.5;

        UnstrGrid result;

        public double Mix
        {
            get { return mix; }
            set { mix = value;
            paramsChanged = true;}
        }

        public PointInterpolator()
        {
            AddInPort("Mesh A", typeof(Zcu.Mve.Visualization.UnstrGrid));
            AddInPort("Mesh B", typeof(Zcu.Mve.Visualization.UnstrGrid));
            AddInPort("Mix", typeof(Zcu.Mve.Numerics.Scalar),false);
            AddOutPort("Output mesh", typeof(Zcu.Mve.Visualization.UnstrGrid));
        }

        public override void Execute()
        {
            if ((!IsAllDataSame()) || paramsChanged)
            {
                if (this.GetInPort("Mix").Link != null)
                {
                    this.mix = (double)((Scalar)this.GetInput("Mix"));
                }
                UnstrGrid meshA = (UnstrGrid)this.GetInput("Mesh A");
                UnstrGrid meshB = (UnstrGrid)this.GetInput("Mesh B");

                if (meshA.Points.Count != meshB.Points.Count)
                {
                    throw new MveException(this, "Interpolating meshes with different number of points. Number of points must be equal.");
                }

                if (meshA.Points.ElemType != meshB.Points.ElemType)
                {
                    throw new MveException(this, "Interpolating meshes with different Point type. Point type must be equal.");
                }

                result = meshA.ShallowCopy();
                UniformDataArray points = new UniformDataArray(meshA.Points.ElemType, meshA.Points.Count);

				double cosMix = ((-Math.Cos(mix * Math.PI))+1)/2;

                for (int i = 0; i < points.Count; i++)
                {
                    IPoint newPoint = (IPoint)meshA.Points[i].DeepCopy();
                    for (int j = 0; j < newPoint.Length; j++)
                    {
						if (interp == interpolator.Linear)
						{
							newPoint[j] = ((IPoint)(meshA.Points[i]))[j] * (1 - mix) + ((IPoint)(meshB.Points[i]))[j] * mix;
						}
						else
						{
							newPoint[j] = ((IPoint)(meshA.Points[i]))[j] * (1 - cosMix) + ((IPoint)(meshB.Points[i]))[j] * cosMix;
						}
                    }
                    points[i] = newPoint;
                }
                result.Points = points;
                SetOutput("Output mesh", result);
                paramsChanged = false;
            }
            else
            {
                SetOutput("Output mesh", result, true);
            }
        }
    }
}