using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;

namespace Examples
{
	class PerlinNoise
	{
		int[] resolutions;
		double[] limits;
		double[][] array;
		int[] multipliers;

		private void setElenemtAt(int[] indices, double value, int coord)
		{			
			int index = 0;
			for (int i=0;i<indices.Length;i++)
			{
				index+=indices[i]*multipliers[i];
			}
			array[coord][index] = value;
		}

		private double getElementAt(int[] indices, int coord)
		{
			int index = 0;
			for (int i = 0; i < indices.Length; i++)
			{
				index += indices[i] * multipliers[i];
			}
			return array[coord][index];
		}

		public double getValueAt(double[] position, int coord)
		{
			int dim = position.Length;
			int size = 1;
			for (int i = 0;i<dim;i++)
				size*=2;
			double[] interpArray = new double[size];
			int[] indices = new int[position.Length];
			for (int i = 0; i < dim; i++)
			{
				indices[i] = (int)(position[i] / (1.0 / resolutions[i]));
			}

			int[] powers = new int[dim];
			powers[0] = 1;

			for (int i = 1; i < dim; i++)
			{
				powers[i] = powers[i - 1] * 2;
			}

			int[] mods = new int[dim];

			for (int i = 0; i < size; i++)
			{
				for (int j = 0; j < dim; j++)
					mods[j] = (((i & powers[j])==0?0:1)) + indices[j];

				interpArray[i] = this.getElementAt(mods, coord);
			}

			for (int i = 0; i < dim; i++)
			{
				double coef = (position[i]-(indices[i]*(1.0/resolutions[i])))/(1.0/resolutions[i]);
				double cosCoef = ((-Math.Cos(coef * Math.PI)) + 1) / 2;
				double[] newArray = new double[interpArray.Length / 2];
				for (int j = 0; j < newArray.Length; j++)
				{
					newArray[j] = interpArray[2 * j] * (1 - cosCoef) + interpArray[2 * j + 1]*cosCoef;					
				}
				interpArray = newArray;
			}
			return interpArray[0];

		}

		public PerlinNoise(int[] resolutions, double[] limits)
		{
			this.resolutions = (int[])resolutions.Clone();
			this.limits = (double[])limits.Clone();
			this.multipliers = new int[resolutions.Length];
			for (int i=0;i<resolutions.Length;i++)
			{
				multipliers[i] = 1;
				for (int j=0;j<i;j++)
				{
					multipliers[i]*=resolutions[j];
				}
			}

			int arraySize = 1;
			for (int i = 0;i<resolutions.Length;i++)
			{
				arraySize*=resolutions[i];
			}

			array = new double[resolutions.Length][];

			for (int i = 0;i<resolutions.Length;i++)
			{
				array[i] = new double[arraySize];
			}

			Random r = new Random();

			for (int coord = 0;coord<resolutions.Length;coord++)
			{
				for (int i = 0; i < arraySize; i++)
				{
					array[coord][i] = (r.NextDouble() - 0.5) * limits[coord];
				}
			}
		}
	}

	class PositionRandomizer : Module
	{
		private double[] limits = { 0.001, 0.001, 0.001, 0.001 };		
		private int seed = 0;
		private bool paramsChanged = true;
		UnstrGrid result = null;

		private int freqCount = 3;

		public int FrequencyCount
		{
			get { return freqCount; }
			set { freqCount = value;
			paramsChanged = true;
		}
		}

			private double decayConst = 0.5;

			public double DecayConstant
			{
				get { return decayConst; }
				set { decayConst = value;
				paramsChanged = true;
			}
			}
	
	

		public int Seed
		{
			get { return seed;}
			set { seed = value;
			paramsChanged = true;
		}
		}	 

		public double[] Limits
		{
			get { return (limits); }
			set { limits = value;
			paramsChanged = true;
		}
		}

		private int[] resolutions = { 25, 25, 25, 25 };

		public int[] Resolutions
		{
			get { return resolutions; }
			set { resolutions = value;
			paramsChanged = true;
		}
		}

        

		public PositionRandomizer()
		{
			AddInPort("Input mesh", typeof(UnstrGrid));
			AddOutPort("Output mesh", typeof(UnstrGrid));
		}        
        
		public override void Execute()
		{
            
			if (paramsChanged || (seed == 0))
			{
				Random random;
				if (seed == 0)
					random = new Random();
				else
					random = new Random(seed);
				UnstrGrid input = (UnstrGrid)GetInput("Input mesh");

				IPoint firstPoint = (IPoint)input.Points[0];
				int compCount = firstPoint.Length;

				double[] sizes = new double[compCount];
				for (int i = 0; i < sizes.Length; i++)
				{
					if (i < limits.Length)
						sizes[i] = limits[i];
					else
						sizes[i] = 0;
				}

				result = input.ShallowCopy();

				BoundingBox bb = input.BoundingBox;
				
				PerlinNoise[] noises = new PerlinNoise[freqCount];

				int[] res = new int[resolutions.Length];
				double[] lims = new double[limits.Length];
				for (int j = 0;j<resolutions.Length;j++)
				{
					res[j] = resolutions[j];
					lims[j] = limits[j];
				}

				for (int i = 0; i < freqCount; i++)
				{
					noises[i] = new PerlinNoise(res, lims);
					for (int j = 0; j < resolutions.Length; j++)
					{
						lims[j] *= decayConst;
						res[j] *= 2;
					}
				}

				UniformDataArray newPoints = new UniformDataArray(input.Points.ElemType, input.Points.Count);
				double[] pointPosition = new double[compCount];
				for (int i = 0; i < newPoints.Count; i++)
				{
					for (int j = 0;j<compCount;j++)
						pointPosition[j] = ((((IPoint)input.Points[i])[j])-bb.MinPoint[j])/(bb.MaxDiag);
					newPoints[i] = input.Points[i].DeepCopy();
					for (int j = 0; j < compCount; j++)
					{
						double noiseValue = 0;
						for (int k = 0; k < freqCount; k++)
						{
							noiseValue += noises[k].getValueAt(pointPosition, j);
						}						
						//((IPoint)newPoints[i])[j] += random.NextDouble() * sizes[j] * 2 - sizes[j];
						((IPoint)newPoints[i])[j] += noiseValue;
					}
				}

				result.Points = newPoints;

				SetOutput("Output mesh", result, false);
				paramsChanged = false;

			}
			else
			{
				SetOutput("Output mesh", result, true);
			}
		}
	}
}
