using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms.Design;
using System.Text;
using System.Reflection;

using Zcu.Mve.Core;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Numerics
{
    [ModuleInfo("Petr Vanek", "Modul for binary operations with Complex type",CategoryType.Filter, CategoryData.Number)]
    [PortInfo("Arg1", "First argument")]
    [PortInfo("Arg2", "Second argument")]
    [PortInfo("Result", "Result of the binary operation")]
    class ComplexBinaryOp : Zcu.Mve.Core.Module
    {
        public class BinOpConverter : StringConverter
        {
            public override bool GetStandardValuesSupported(ITypeDescriptorContext context) { return true; }
            public override bool GetStandardValuesExclusive(ITypeDescriptorContext context) { return true; }

            public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
            {
                List<string> functions = new List<string>();
                MethodInfo[] methods = typeof(Complex).GetMethods(BindingFlags.Public | BindingFlags.Static);
                for (int i = 0; i < methods.Length; i++)
                {
                    if (methods[i].ReturnType != typeof(Complex)) continue;
                    ParameterInfo[] param = methods[i].GetParameters();
                    if (param.Length != 2) continue;
                    if (param[0].ParameterType != typeof(Complex)) continue;
                    if (param[1].ParameterType != typeof(Complex)) continue;
                    functions.Add(methods[i].Name);
                }
                return new StandardValuesCollection(functions);
            }
        }

        delegate Complex func(Complex a, Complex b);
        func Func;

        private string operation;

        [Description("Binary operator.")]
        [TypeConverter(typeof(BinOpConverter))]
        public string Operation
        {
            get { return operation; }
            set
            {
                operation = value;
                MethodInfo method = typeof(Complex).GetMethod(operation, new Type[] { typeof(Complex), typeof(Complex) });
                Func = (func)Delegate.CreateDelegate(typeof(func), method);
            }
        }

        /// <summary>
        /// result kept for the next run of the module
        /// </summary>
        private Complex result;

        /// <summary>
        /// Adds module input and output ports.
        /// </summary>
        public ComplexBinaryOp()
        {
            Operation = "op_Addition";
            AddInPort("Arg1", typeof(Complex));
            AddInPort("Arg2", typeof(Complex));
            AddOutPort("Result", typeof(Complex));
        }

        /// <summary>
        /// Performs the multiplication. If no inputs were chnaged since the last run, 
        /// then the last result is returned immediately with the DataSame flag set.
        /// </summary>
        public override void Execute()
        {
            if (IsAllDataSame() && IsAllUserSettingsSame())
            {
                this.WriteDebug("Skip execution", "nothing changed");
                this.ProgressInfo(1);
                SetOutput("Result", result, true);
            }
            else
            {
                Complex a = (Complex)GetInput("Arg1");
                Complex b = (Complex)GetInput("Arg2");

                result = Func(a, b);
                this.ProgressInfo(1);
                SetOutput("Result", result);
            }
        }
    }
}
