using System;
using Zcu.Mve.Core;
using System.ComponentModel;

namespace Zcu.Mve.Numerics
{
    /// <summary>
    /// A source module for Complex
    /// </summary>
    [ModuleInfo("Libor Va", "A source module for Complex",CategoryType.Source, CategoryData.Number)]
    [PortInfo("Complex", "Resulting Complex")]
    public class ComplexSource : Module
    {
        private Random r = new Random();
        private Complex output;
        private double real;
        private double imag;
        private bool randomize = false;
        private int randomSeed = 0;
        private bool forceExecution = false;

        /// <summary>
        /// Real part of complex number that will be set as output.
        /// </summary>        
        [Description("Real part of complex number that will be used as output of the module")]
        public double Real
        {
            get { return (real); }
            set { real = value; }
        }

        /// <summary>
        /// Imaginary part of complex number that will be set as output.
        /// </summary>        
        [Description("Imaginary part of complex number that will be used as output of the module")]
        public double Imag
        {
            get { return (imag); }
            set { imag = value; }
        }

        /// <summary>
        /// Generate random Integer. 
        /// </summary>
        [Description("Generate random Integer")]
        [Category("Random")]
        public bool Randomize
        {
            get { return randomize; }
            set { randomize = value; }
        }

        /// <summary>
        /// Random seed. 
        /// </summary>
        [Description("Random seed")]
        [Category("Random")]
        public int RandomSeed
        {
            get { return randomSeed; }
            set
            {
                randomSeed = value;
                if (randomSeed < 0) r = new Random();
                else r = new Random(randomSeed);
            }
        }

        /// <summary>
        /// Generate random Integer. 
        /// </summary>
        [Description("Force execution when nothing changed.")]
        [Category("Execution")]
        public bool ForceExecution
        {
            get { return forceExecution; }
            set { forceExecution = value; }
        }


        /// <summary>
        /// Default constructor.
        /// </summary>
        public ComplexSource()
        {
            AddOutPort("Complex", typeof(Complex));
        }

        /// <summary>
        /// Checks wether value property was changed and sets the output accordingly.
        /// If value was not changed since the last run then dataSame is set to true.
        /// </summary>
        public override void Execute()
        {
            if (IsAllUserSettingsSame() && !forceExecution)
            {
                this.WriteDebug("Skip execution", "nothing changed");
                this.ProgressInfo(1);
                this.SetOutput("Complex", output, true);
            }
            else
            {
                if (randomize)
                {
                    output = new Complex(r.NextDouble(),r.NextDouble());

                    this.ProgressInfo(1);
                    this.SetOutput("Integer", output, false);
                }
                else
                {
                    output = new Complex(this.real, this.imag);
                    this.ProgressInfo(1);
                    this.SetOutput("Complex", output, false);
                }
            }
        }

    }
}
