using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms.Design;
using System.Text;
using System.Reflection;

using Zcu.Mve.Core;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Numerics
{
    [ModuleInfo("Petr Vanek", "Modul for binary operations with Integer type", CategoryType.Filter, CategoryData.Number)]
    [PortInfo("Arg1", "First argument")]
    [PortInfo("Arg2", "Second argument")]
    [PortInfo("Result", "Result of the binary operation")]
    class IntegerBinaryOp : Zcu.Mve.Core.Module
    {
        public class BinOpConverter : StringConverter 
        {
            public override bool GetStandardValuesSupported(ITypeDescriptorContext context) { return true; }
            public override bool GetStandardValuesExclusive(ITypeDescriptorContext context) { return true; }

            public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
            {
                List<string> functions = new List<string>();
                MethodInfo[] methods = typeof(Integer).GetMethods(BindingFlags.Public | BindingFlags.Static);
                for (int i = 0; i < methods.Length; i++)
                {
                    if (methods[i].ReturnType!=typeof(Integer)) continue;
                    ParameterInfo[] param = methods[i].GetParameters();
                    if(param.Length!=2) continue;
                    if(param[0].ParameterType!=typeof(Integer)) continue;
                    if(param[1].ParameterType!=typeof(Integer)) continue;
                    functions.Add(methods[i].Name);
                }
                return new StandardValuesCollection(functions);
            }
        }

        delegate Integer func(Integer a, Integer b);
        func Func;

        private string operation;

        [Description("Binary operator.")]
        [TypeConverter(typeof(BinOpConverter))]
        public string Operation
        {
            get { return operation; }
            set
            {
                operation = value;
                MethodInfo method = typeof(Integer).GetMethod(operation, new Type[] { typeof(Integer), typeof(Integer) });
                Func = (func)Delegate.CreateDelegate(typeof(func), method);
            }
        }

        /// <summary>
        /// result kept for the next run of the module
        /// </summary>
        private Integer result;

        /// <summary>
        /// Adds module input and output ports.
        /// </summary>
        public IntegerBinaryOp()
		{
            Operation = "op_Addition";
			AddInPort("Arg1",typeof(Integer));
			AddInPort("Arg2",typeof(Integer));
			AddOutPort("Result", typeof(Integer));
		}
        
        /// <summary>
        /// Performs the multiplication. If no inputs were chnaged since the last run, 
        /// then the last result is returned immediately with the DataSame flag set.
        /// </summary>
        public override void Execute()
        {
            if (IsAllDataSame()&&IsAllUserSettingsSame())
            {
                this.WriteDebug("Skip execution", "nothing changed");
                this.ProgressInfo(1);
                SetOutput("Result", result, true);
            }
            else
            {
                Integer a = (Integer)GetInput("Arg1");
                Integer b = (Integer)GetInput("Arg2");

                result = Func(a, b);
                this.ProgressInfo(1);
                SetOutput("Result", result);
            }
        }
    }
}
