using System;
using System.Text;
using System.ComponentModel;
using System.IO;
using System.Collections.Generic;
using System.Runtime.InteropServices;

using Zcu.Mve.Core;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Strings
{
	class DirectoryLister:Module
    {
        [DllImport("shlwapi.dll", CharSet = CharSet.Unicode)]
        private static extern int StrCmpLogicalW(string psz1, string psz2);

        private static int NaturalStringCompare(string x, string y)
        {
            return StrCmpLogicalW(x, y);
        }


        private string[] _files = null;

		private string _dirName = ".";
		private bool _reload = false;

		private MveString _res;
        private MveString _resWithout;
        private Integer _count;

		[Editor(typeof(System.Windows.Forms.Design.FolderNameEditor), typeof(System.Drawing.Design.UITypeEditor))]
		public string DirName
		{
			get { return _dirName; }
            set { _dirName = value; }
		}

		public bool Reload
		{
			get { return _reload; }
			set { _reload = value; }
		}


		public DirectoryLister()
		{
            AddInPort("BaseDirectory", typeof(MveString), false);
			AddInPort("FileNumber", typeof(Integer));
            AddOutPort("FileNameWithPath", typeof(MveString));
            AddOutPort("FileName", typeof(MveString));
		}


		public override void Execute()
		{
            if (_reload || !IsAllUserSettingsSame() || !IsAllDataSame())
            {
                if (GetInPort("BaseDirectory").Link != null)
                {
                    _dirName = (string)(MveString)GetInput("BaseDirectory");
                }

                if (_reload || !IsAllUserSettingsSame() || _files == null)
                {
                    _files = Directory.GetFiles(Path.GetFullPath(_dirName));
                    _count = _files.Length;
                    Array.Sort(_files, NaturalStringCompare);
                }

                int number = (Integer)GetInput("FileNumber");
                if (number >= _files.Length)
                {
                    throw new MveException(this, "The directory contains less than " + number + "files.");
                }

                _res = _files[number];
                _resWithout = _files[number].Substring(_files[number].LastIndexOf('\\')+1);
                SetOutput("FileName", _resWithout, false);
                SetOutput("FileNameWithPath", _res, false);
                WriteLog("Directory", _dirName);
                WriteLog("File Name With Path", _res.Value);
                WriteLog("File Name", _resWithout.Value);
            }
            else
            {
                SetOutput("FileName", _res, true);
            }

		}
	}
}
