﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

using Zcu.Mve.Core;

namespace Zcu.Mve.Strings
{
    /// <summary>
    /// A module that performs a regular expression replace on the input string.
    /// </summary>
    [ModuleInfo("Oldřich Petřík", "Performs a regular expression replace on the input string.", CategoryType.Filter, CategoryData.String)]
    [PortInfo("InString", "The input string.")]
    [PortInfo("OutString", "The replaced string.")]
    public class RegExReplacer : Module
    {
        private MveString _res = null;

        private string _inPat = "(.*)";
        private string _outPat = "$1";
        private Regex _regex;

        /// <summary>A regular expression that matches the input string.</summary>
        public string MatchPattern
        {
            get { return _inPat; }
            set { CheckAssignRegex(value); }
        }

        /// <summary>A replacement string for the matched input string.</summary>
        public string ReplacePattern
        {
            get { return _outPat; }
            set { _outPat = value; }
        }

        /// <summary>
        /// Module constructor.
        /// </summary>
        public RegExReplacer()
        {
            AddInPort("InString", typeof(MveString), true);
            AddOutPort("OutString", typeof(MveString));

            _regex = new Regex(_inPat, RegexOptions.Compiled);
        }

        private void CheckAssignRegex(string value)
        {
            Regex res;
            try
            {
                res = new Regex(value, RegexOptions.Compiled);
                _regex = res;
                _inPat = value;
            }
            catch (ArgumentException)
            {
                System.Windows.Forms.MessageBox.Show(
                    "The specified match pattern cannot be compiled!", Name,
                    System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Executes the replacement process.
        /// </summary>
        public override void Execute()
        {
            if (_res != null && IsAllDataSame() && IsAllUserSettingsSame())
            {
                SetOutput("OutString", _res, true);
            }
            else
            {
                string input = (MveString)GetInput("InString");
                _res = _regex.Replace(input, _outPat);
                SetOutput("OutString", _res, false);
            }
        }
    }
}
