using System;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Testing
{
    /// <summary>
    /// Acceleration simulation module
    /// </summary>
    public class GravityAcceleration : Zcu.Mve.Core.Module
    {
        Vector2D acceleration1 = new Vector2D();
        Vector2D acceleration2 = new Vector2D();
        double oveckin = 0.0001;

        /// <summary>
        /// Constructor.
        /// </summary>
        public GravityAcceleration()
        {
            AddInPort("oveckin", typeof(Scalar), false);
            AddInPort("pos1", typeof(Vector2D));
            AddInPort("pos2", typeof(Vector2D));
            AddInPort("weight1", typeof(Scalar));
            AddInPort("weight2", typeof(Scalar));
            AddOutPort("acceleration1", typeof(Vector2D));
            AddOutPort("acceleration2", typeof(Vector2D));
        }

        /// <inheritdoc />
        public override void Execute()
        {
            double m1 = ((Scalar)GetInput("weight1")).Value;
            double m2 = ((Scalar)GetInput("weight2")).Value;
            Vector2D p1 = (Vector2D)GetInput("pos1");
            Vector2D p2 = (Vector2D)GetInput("pos2");

            Scalar oveckinScalar = (Scalar)GetInput("oveckin");
            //if (oveckinScalar != null)
            oveckin = oveckinScalar.Value;

            // vzdalenost v jednotlivych osach
            double dx = p1.X - p2.X;
            double dy = p1.Y - p2.Y;

            // vzdalenost objektu
            double r = Math.Sqrt(dx * dx + dy * dy);

            // pusobici sila
            double F;
            if (r == 0.0)
                F = 0;
            else
                F = m1 * m2 / r;

            acceleration1.X = (-dx / r) * F / m1 * oveckin;
            acceleration1.Y = (-dy / r) * F / m1 * oveckin;

            acceleration2.X = (dx / r) * F / m2 * oveckin;
            acceleration2.Y = (dy / r) * F / m2 * oveckin;

            SetOutput("acceleration1", acceleration1);
            SetOutput("acceleration2", acceleration2);
        }
    }
}
