using System;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.ComponentModel;


namespace Zcu.Mve.Testing
{
    /// <summary>
    /// Generates an array of random 2D points.
    /// </summary>
    class RandomPoint2DGenerator : Module
    {
        private static Random rnd = new Random(0);

        /// <summary>
        /// Constructor.
        /// </summary>
        public RandomPoint2DGenerator()
        {
            AddOutPort("Points", typeof(UniformDataArray));
        }

        private int pointCount;

        /// <summary>
        /// Number of points to be generated.
        /// Must be greater than zero.
        /// </summary>
        [Description("Number of points to be generated. Must be greater than zero.")]
        public int PointCount
        {
            get { return pointCount; }
            set
            {
                if (value > 0)
                    pointCount = value;
                else
                {
                    throw new MveException(this, "Random values count cannot be lower nor equal to zero.");
                }
            }
        }

        /// <summary>
        /// generates the random numbers
        /// </summary>
        public override void Execute()
        {
            UniformDataArray output = new UniformDataArray(typeof(Point2D), pointCount);
            for (int i = 0; i < pointCount; i++)
            {
                output[i] = new Point2D(rnd.NextDouble(), rnd.NextDouble());
            }
            SetOutput("Points", output);
        }
    }
}
