using System;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;

namespace Zcu.Mve.Testing
{
    /// <summary>
    /// Converts MveArray to RegGridND.
    /// </summary>
    public class MveArray2RegGridND : Module
    {
        RegGridND output = null;

        bool cellAttrs = false;

        string attrName = "data";

        /// <summary>
        /// Gets/sets the name of the output attribute.
        /// </summary>
        public string AttrName
        {
            get { return (attrName); }
            set
            {
                if (value == "")
                {
                    throw new MveException(this, "Attribute name cannot be an empty string.");
                }
                else
                    attrName = value;
            }
        }

        /// <summary>
        /// Add the values as a cell attribute (or a point attribute)?
        /// </summary>
        public bool CellAttrs
        {
            get { return (cellAttrs); }
            set { cellAttrs = value; }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public MveArray2RegGridND()
        {
            AddInPort("Input MveArray", typeof(MveArray));
            AddOutPort("Output RegGridND", typeof(RegGridND));
        }

        /// <inheritdoc />
        public override void Execute()
        {
            if ((output == null) || (!IsDataSame("Input MveArray")))
            {
                MveArray input = (MveArray)GetInput("Input MveArray");

                int[] lengths = input.Lengths;

                if (!cellAttrs)
                {
                    for (int i = 0; i < lengths.Length; i++)
                        lengths[i]--;
                }

                output = new RegGridND(lengths);

                if (cellAttrs)
                {
                    output.AddCellAttrs(attrName, input);
                }
                else
                {
                    output.AddPointAttrs(attrName, input);
                }

                SetOutput("Output RegGridND", output);
            }
            else
            {
                SetOutput("Output RegGridND", output, true);
            }
        }
    }
}
