using System;
using System.Collections.Generic;
using Zcu.Mve.Visualization;
using System.Collections;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Represents arrow glyph
    /// </summary>
    class Arrow : Glyph3D
    {
        #region  Variables Declaration
        private static int defTipResolution = 5;
        /// <summary>
        /// Default arrow tip resolution
        /// </summary>
        public static int DefTipResolution
        {
            get { return defTipResolution; }
        }
        private static double defTipRadius = 0.005;
        /// <summary>
        /// Default arrow tip radius
        /// </summary>
        public static double DefTipRadius
        {
            get { return defTipRadius; }
        }

        private static double defArrowLength = 0.1;
        /// <summary>
        /// Default arrow length
        /// </summary>
        public static double DefArrowLength
        {
            get { return defArrowLength; }
        }
        private static double defTipLength = 0.3;
        /// <summary>
        /// Default arrow tip relative length
        /// </summary>
        public static double DefTipLength
        {
            get { return defTipLength; }
        }
        private static int defShaftResolution = 5;
        /// <summary>
        /// Default arrow shaft resolutin
        /// </summary>
        public static int DefShaftResoluion
        {
            get { return defShaftResolution; }
        }
        private static double defShaftRadius = 0.01;
        /// <summary>
        /// Default arrow tip radius
        /// </summary>
        public static double DefShaftRadius
        {
            get { return defShaftRadius; }
        }
        #endregion

        /// <summary>
        /// Constructor, creates arrow points and cells. Arrow glyph is composit of Cone and Cylinder glyph.
        /// </summary>
        /// <param name="tipResoulution"> resolution of arrow tip</param>
        /// <param name="tipRadius">radius of arrow tip</param>
        /// <param name="tipLength">relative lenght of arrow tip (0..1) </param>
        /// <param name="shaftReslution">resolution of arrow shaft</param>
        /// <param name="shaftRadius">radius of arrow shaft</param>
        /// <param name="arrowLength">length of arrow </param>
        public Arrow(int tipResoulution, double tipRadius, double tipLength, int shaftReslution, double shaftRadius, double arrowLength)
        {
            if (tipLength > 1 || tipLength < 0)
                throw new MveException(this, " Incorect tip length:" + tipLength.ToString() + " , tiplength must be float number between 0 and 1");
            double tipHeight = arrowLength * tipLength;// computes 
            double shaftHeigth = arrowLength - tipHeight;
            Cone shaft = new Cone(shaftReslution, shaftHeigth, shaftRadius, true, new Center(0, 0, tipHeight));
            Cylinder tip = new Cylinder(tipResoulution, tipHeight, tipRadius, true, new Center(0, 0, tipHeight / 2));
            glyphPoints.AddRange(shaft.GlyphPoints);// add shaft points to arrow points list
            int offset = shaft.GlyphPoints.Count;// add shaft cells to arrow cells list
            for (int i = 0; i < shaft.GlyphCells.Count; i++)
            {
                glyphCells.Add(shaft.GlyphCells[i]);
            }

            glyphPoints.AddRange(tip.GlyphPoints);// add tip points to arrow points list

            for (int i = 0; i < tip.GlyphCells.Count; i++)// add tip cells to arrow cells list
            {
                Triangle tmp = (Triangle)tip.GlyphCells[i];
                // we have to use offset because shaft points an cells are allready in the lists
                glyphCells.Add(new Triangle(tmp.V1 + offset, tmp.V2 + offset, tmp.V3 + offset));
            }
        }
    }
}
