using System;
using System.Collections.Generic;
using Zcu.Mve.Visualization;
using System.Collections;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Represents box glyph
    /// </summary>
    class Box : Glyph3D
    {
        #region Variables Declaration
        // default values of box glyph attributes
        private static double defXLenght = 0.01;
        /// <summary>
        /// Default box xlenght
        /// </summary>
        public static double DefXLenght
        {
            get { return defXLenght; }
        }

        private static double defYLenght = 0.01;
        /// <summary>
        /// Default box y lenght
        /// </summary>
        public static double DefYLenght
        {
            get { return defYLenght; }
        }

        private static double defZLenght = 0.01;
        /// <summary>
        /// Default box z lenght
        /// </summary>
        public static double DefZLenght
        {
            get { return defZLenght; }
        }

        private static Center defCenter = new Center(0, 0, 0);
        /// <summary>
        /// Default box center
        /// </summary>
        public static Center DefCenter
        {
            get { return defCenter; }
        }
        #endregion

        /// <summary>
        /// default constructor
        /// </summary>
        /// <param name="xLenght">lenght of x edge of the box</param>
        /// <param name="yLenght">lenght of x edge of the box</param>
        /// <param name="zLenght">lenght of x edge of the box</param>
        /// <param name="center">center of the box</param>
        public Box(double xLenght, double yLenght, double zLenght, Center center)
        {
            double xhalf = xLenght / 2.0;
            double yhalf = yLenght / 2.0;
            double zhalf = zLenght / 2.0;
            glyphPoints.Add(new Point3D(xhalf + center.x, -yhalf + center.y, zhalf + center.z));
            glyphPoints.Add(new Point3D(-xhalf + center.x, -yhalf + center.y, zhalf + center.z));
            glyphPoints.Add(new Point3D(-xhalf + center.x, yhalf + center.y, zhalf + center.z));
            glyphPoints.Add(new Point3D(xhalf + center.x, yhalf + center.y, zhalf + center.z));

            glyphPoints.Add(new Point3D(xhalf + center.x, -yhalf + center.y, -zhalf + center.z));
            glyphPoints.Add(new Point3D(-xhalf + center.x, -yhalf + center.y, -zhalf + center.z));
            glyphPoints.Add(new Point3D(-xhalf + center.x, yhalf + center.y, -zhalf + center.z));
            glyphPoints.Add(new Point3D(xhalf + center.x, yhalf + center.y, -zhalf + center.z));

            for (int i = 0; i < 3; i++)
            {
                glyphCells.Add(new Triangle(i, i + 4, i + 1));
                glyphCells.Add(new Triangle(i + 1, i + 4, i + 5));
            }

            glyphCells.Add(new Triangle(3, 7, 0));
            glyphCells.Add(new Triangle(0, 7, 4));

            glyphCells.Add(new Triangle(4, 7, 5));
            glyphCells.Add(new Triangle(5, 7, 6));

            glyphCells.Add(new Triangle(1, 5, 0));
            glyphCells.Add(new Triangle(1, 2, 5));
        }
    }
}
