using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Gui for box glyph
    /// </summary>
    public partial class BoxGui : GlyphGui
    {
        #region Variables Declaration
        // box glyph attributes
        private double xLenght = Box.DefXLenght;
        /// <summary>
        /// x lenght of the box
        /// </summary>
        public double XLenght
        {
            get { return xLenght; }
        }

        private double yLenght = Box.DefYLenght;
        /// <summary>
        /// y lenght of the box
        /// </summary>
        public double YLenght
        {
            get { return yLenght; }
        }

        private double zLenght = Box.DefZLenght;
        /// <summary>
        /// z lenght of the box
        /// </summary>
        public double ZLenght
        {
            get { return zLenght; }
        }

        private Center center = Box.DefCenter;
        /// <summary>
        /// Center of the box
        /// </summary>
        public Center Center
        {
            get { return center; }
        }
        #endregion

        /// <summary>
        /// Default constructor
        /// </summary>
        public BoxGui()
        {
            InitializeComponent();
            InitGuiValues();
        }

        /// <summary>
        /// Return an array of currnet box glyph paramatres
        /// </summary>
        /// <returns>An array of currnet box glyph paramatres</returns>
        public override List<string> GetParametrs()
        {
            List<string> paramatrs = new List<string>();
            paramatrs.Add(xLenght.ToString(Globals.nfi));
            paramatrs.Add(yLenght.ToString(Globals.nfi));
            paramatrs.Add(zLenght.ToString(Globals.nfi));
            paramatrs.Add(center.x.ToString(Globals.nfi));
            paramatrs.Add(center.y.ToString(Globals.nfi));
            paramatrs.Add(center.z.ToString(Globals.nfi));
            return paramatrs;
        }

        /// <summary>
        /// Sets box gui attributes according to values in input array
        /// </summary>
        /// <param name="parameters"> new gui values</param>
        public override void SetParametrs(List<string> parameters)
        {
            xLenght = Double.Parse(parameters[0], Globals.nfi);
            yLenght = Double.Parse(parameters[1], Globals.nfi);
            zLenght = Double.Parse(parameters[2], Globals.nfi);
            center.x = Double.Parse(parameters[3], Globals.nfi);
            center.y = Double.Parse(parameters[4], Globals.nfi);
            center.z = Double.Parse(parameters[5], Globals.nfi);
            InitGuiValues();
        }

        /// <summary>
        /// Sets box attributes to gui components
        /// </summary>
        private void InitGuiValues()
        {
            xLengthTBox.Text = xLenght.ToString(Globals.nfi);
            yLengthTBox.Text = yLenght.ToString(Globals.nfi);
            zLengthTBox.Text = zLenght.ToString(Globals.nfi);
            centerXTBox.Text = center.x.ToString(Globals.nfi);
            centerYTBox.Text = center.y.ToString(Globals.nfi);
            centerZTBox.Text = center.z.ToString(Globals.nfi);
        }

        private void xLengthTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeded = ParseDouble(ref xLenght, xLengthTBox.Text, "Lengths must be positive doubles", false);
            if (!succeded)
            {
                xLengthTBox.Text = Box.DefXLenght.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }

        }

        private void yLengthTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeded = ParseDouble(ref yLenght, yLengthTBox.Text, "Lengths must be positive doubles", false);
            if (!succeded)
            {
                yLengthTBox.Text = Box.DefYLenght.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void zLengthTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeded = ParseDouble(ref zLenght, zLengthTBox.Text, "Lengths must be positive doubles", false);
            if (!succeded)
            {
                zLengthTBox.Text = Box.DefZLenght.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerXTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeded = ParseDouble(ref center.x, centerXTBox.Text, "Coordinates must be positive integers", true);
            if (!succeded)
            {
                centerXTBox.Text = Box.DefCenter.x.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }

        }

        private void centerYTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeded = ParseDouble(ref center.y, centerYTBox.Text, "Coordinates must be positive integers", true);
            if (!succeded)
            {
                centerYTBox.Text = Box.DefCenter.y.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerZTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeded = ParseDouble(ref center.z, centerZTBox.Text, "Coordinates must be positive integers", true);
            if (!succeded)
            {
                centerZTBox.Text = Box.DefCenter.z.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }
    }
}
