using System;
using System.Collections.Generic;
using Zcu.Mve.Visualization;
using System.Collections;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Represent cone g;yph
    /// </summary>
    class Cone : Glyph3D
    {
        #region Variables Declaration
        // default cone atribute values
        private static double defRadius = 0.01;
        /// <summary>
        /// Default cone radius
        /// </summary>
        public static double DefRadius
        {
            get { return defRadius; }
        }

        private static bool defCapping = true;
        /// <summary>
        /// Default capping value
        /// </summary>
        public static bool DefCapping
        {
            get { return defCapping; }
        }

        private static double defHeigth = 0.1;
        /// <summary>
        /// Default Cone Height value
        /// </summary>
        public static double DefHeigth
        {
            get { return defHeigth; }
        }

        private static int defResolution = 6;
        /// <summary>
        /// Default Cone resolution
        /// </summary>
        public static int DefResolution
        {
            get { return defResolution; }
        }

        private static Center defCenter = new Center(0, 0, 0);
        /// <summary>
        /// Default Cone center
        /// </summary>
        public static Center DefCenter
        {
            get { return defCenter; }
        }
        #endregion

        /// <summary>
        /// Default Constructor
        /// </summary>
        /// <param name="resolution"> cone resolution</param>
        /// <param name="height"> cone height</param>
        /// <param name="radius"> cone radius</param>
        /// <param name="capping"> if true cone bottom capping is used</param>
        /// <param name="center"> center of the glyph</param>
        public Cone(int resolution, double height, double radius, bool capping, Center center)
        {
            glyphPoints.Add(new Point3D(center.x, center.y, center.z + height / 2.0));
            double phi = 0;
            double phiStep = 2 * Math.PI / resolution;
            double heightTmp = 0;
            for (int i = 0; i < resolution; i++)
            {
                glyphPoints.Add(new Point3D(radius * Math.Sin(phi) + center.x, radius * Math.Cos(phi) + center.y, heightTmp + center.z));
                phi += phiStep;
            }

            for (int i = 1; i < resolution; i++)
            {
                glyphCells.Add(new Triangle(0, i, i + 1));
            }
            glyphCells.Add(new Triangle(0, resolution, 1));

            if (capping)// add capping triangles
            {
                glyphPoints.Add(new Point3D(center.x, center.y, center.z));
                int lastIndex = glyphPoints.Count - 1;
                for (int i = 1; i < resolution; i++)
                {
                    glyphCells.Add(new Triangle(lastIndex, i + 1, i));
                }
                glyphCells.Add(new Triangle(lastIndex, 1, resolution));
            }
        }
    }
}
