using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.Diagnostics;

namespace Zcu.Mve.Visualization.Processing
{
  /// <summary>
  /// A general class that composes a new uniformDataArray. Any Dataobject can be attached to the input port, causing further ports of the same type to appear.
  /// </summary>
  [PortInfo("Head", "First element of the array shoud be connected to this port, causing further ports to appear")]
  [PortInfo("Result", "Resulting UniformDataArray")]
  [ModuleInfo("Libor Va", "A general class that composes a new uniformDataArray. Any Dataobject can be attached to the input port, causing further ports of the same type to appear.", CategoryType.Generic, CategoryData.Generic, IconName = "UniformDataArrayComposer.ico")]
  class UniformDataArrayComposer : Module
  {
    private int portcount = 1;

    private Port headPort;

    private Type elemType;

    private UniformDataArray result = null;

    /// <summary>
    /// default constructor. Only one input port is created here, subsequent ports are added when links are connected to the input.
    /// </summary>
    public UniformDataArrayComposer()
    {
      AddInPort("Head", typeof(IDataObject));
      AddOutPort("Result", typeof(UniformDataArray));

      headPort = this.GetInPort("Head");

      this.PortConnected += new PortConnectedEventHandler(UnifrmDataArrayComposer_OnConnectEvent);
      this.PortDisconnecting += new PortDisconnectingEventHandler(UniformDataArrayComposer_OnDisconnectEvent);
    }

    void UniformDataArrayComposer_OnDisconnectEvent(object source, OutPort portFrom, InPort portTo, out bool allowed)
    {
      allowed = true;
      if (portcount != 1)
      {
        if (portTo.Name != ("E" + (portcount - 2)))
        {
          allowed = false;
        }
        else
        {
          RemoveInPort("E" + (portcount - 1));
          portcount--;
        }
      }
    }

    void UnifrmDataArrayComposer_OnConnectEvent(object source, OutPort portFrom, InPort portTo)
    {
      if (portTo == headPort)
      {
        elemType = portFrom.PortType;
      }
      AddInPort("E" + portcount, elemType, false);
      portcount++;
    }

    /// <summary>
    /// Performs the uniformDataArray composition. If no inputs were changed since the last run, then the result of the last run is returned immediately.
    /// </summary>
    public override void Execute()
    {
      if (!IsAllDataSame() || (result == null))
      {
        //Debug.WriteLine("UniformDataArrayComposer - update");

        result = new UniformDataArray(elemType, portcount - 1);
        result[0] = (IDataObject) GetInput("Head");
        for (int i = 1; i < (portcount - 1); i++)
        {
          result[i] = (IDataObject) GetInput("E" + i);
        }
        SetOutput("Result", result);
      }
      else
      {
        //Console.WriteLine("tedkins sem NEupdatoval...");
        SetOutput("Result", result, true);
      }
    }
  }
}
